﻿/********************************************************************
  Copyright (c) 2025 此乃刘同学.www.liustu.com.cn
  FileName : createtable.c
  Author   : 刘斌
  Version  : V1.0
  Date     : 2025-3-31
  Note     : C仿写sqlite3-嵌入式轻量级数据库
  History  : 2025-4-2 	V1.0 		完成初步编辑
********************************************************************/
#include "createtable.h"

/**
  * @brief  创建表
  * @param 	dbname: 数据库名,tablename: 表名
  * @retval 0:成功
  */
int Create_Table(char *dbname, char *tablename){
    char line_str[256];
    char cmd[100]; 
    int start_len = (strlen(dbname) + 7);
    char name[20];
    char type[50];
    int name_count;
    int state = 0;
    int default_num = 0;

    /* 开头格式存储 */
    char *table_temp = (char *)malloc(strlen(tablename) + 6) ;
    if(!table_temp) {
        printf("注意：内存分配失败\r\n");
    }
    sprintf(table_temp, "TABLE %s", tablename);
    //printf("%s\n",table_temp);

    /* 跳转到数据库存放的路径下 */
    chdir("sqlfile");

     /* 打开或创建数据库 */
    FILE *dbfile = fopen(dbname, "r+");

    /* 逐行读取 */
    while(fgets(line_str, sizeof(line_str), dbfile)){
        /* 当出现 tablename时 */
        if(strstr(line_str, table_temp) != NULL) {
            printf("ERROR : 存在相同名称表\n");
             /* 关闭文件 */
            fclose(dbfile);
            dbfile = NULL;
            return 1;
        }
    }

    /* 将文件指针重置到文件末尾，以便追加数据 */
    fseek(dbfile, 0, SEEK_END);

    /* 存放表名 */
    fprintf(dbfile, "TABLE %s :\n", tablename);

    /* 输入表的数据 */
    while(1){
        int count = start_len;
        while(count--)
            printf(" ");
        printf("> ");
        if (fgets(cmd, sizeof(cmd), stdin) == NULL) {
            break; // 读取输入失败或EOF
        }

        /* 去掉换行符 */
        cmd[strcspn(cmd, "\n")] = 0;

        /* 输入结束符 */
        if(strstr(cmd, ");"))
            break;

        /* 提取出变量名 */
        sscanf(cmd, "%[^ ]", name);  
        name_count = strlen(name);

        /* 提取出变量类型 */
        char *type_start = cmd + name_count; 
        while (*type_start == ' ') { 
            type_start++;
        }
        sscanf(type_start, "%s", type);
        
        /* 判断输入类型中含有的约束 */
        state = 0; // 重置状态
        if(strstr(cmd, "NOT NULL")){      // 不能为NULL
            state |= NOT_NULL;               // state低1位置1
        }
        if(strstr(cmd, "DEFAULT")){          // 为空的时候存放默认值
            state |= DEFAULT;                // state低2位置1
            /* 这里需要提取默认值，假设默认值是整数 */
            char *default_start = strstr(cmd, "DEFAULT") + 8;
            sscanf(default_start, "%d", &default_num);
        }
        if(strstr(cmd, "UNIQUE")){           // 不能为相同
            state |= UNIQUE;                 // state低3位置1
        }
        if(strstr(cmd, "PRIMARY KEY")){      // 不能为相同
            state |= NOT_NULL;            // state低4位置1
            state |= PRIMARY_Key;
        }

        /* 将变量名、类型和约束写入文件 */
        fprintf(dbfile, "%s %s ", name, type);
        if (state & NOT_NULL) fprintf(dbfile, "NOT NULL ");
        if (state & DEFAULT) fprintf(dbfile, "DEFAULT %d ", default_num);
        if (state & UNIQUE) fprintf(dbfile, "UNIQUE ");
        if (state & PRIMARY_Key) fprintf(dbfile, "PRIMARY KEY ");
        fprintf(dbfile, "\n");
    }
    
    fprintf(dbfile, "END\n\n");


    /* 关闭文件 */
    fclose(dbfile);
    dbfile = NULL;
    
    return 0;
}