﻿/********************************************************************
  Copyright (c) 2025 此乃刘同学.www.liustu.com.cn
  FileName : delete.c
  Author   : 刘斌
  Version  : V1.0
  Date     : 2025-3-31
  Note     : C仿写sqlite3-嵌入式轻量级数据库
  History  : 2025-4-2 	V1.0 		完成初步编辑
********************************************************************/
#include "delete.h"
#include <unistd.h>

/**
  * @brief  删除表
  * @param 	dbname: 数据库名,cmd: 用户输入命令
  * @retval 0:成功
  */
int Delete_Table(char *dbname, char *cmd) {
    char line_str[256];
    fpos_t pos_start;  // 保留这个变量，后续有用

    /* 检查结束符 */
    if (strchr(cmd, ';') == NULL) {
        printf("错误：缺少\";\"结束符\r\n");
        return -1;
    }

    /* 提取表名 */
    char *table_start = strstr(cmd, "TABLE");
    if (!table_start) {
        printf("语法错误：缺少TABLE关键字\r\n");
        return -1;
    }
    table_start += strlen("TABLE ");
    char *table_end = strchr(table_start, ';');
    if (!table_end) {
        printf("语法错误：缺少分号\r\n");
        return -1;
    }
    size_t table_len = table_end - table_start;
    char *tablename = malloc(table_len + 1);
    if (!tablename) {
        printf("内存分配失败\r\n");
        return -1;
    }
    strncpy(tablename, table_start, table_len);
    tablename[table_len] = '\0';

    /* 打开数据库文件 */
    chdir("sqlfile");
    FILE *dbfile = fopen(dbname, "r+");
    if (!dbfile) {
        printf("无法打开数据库文件: %s\r\n", dbname);
        free(tablename);
        return -1;
    }

    /* 查找表位置 */
    int found = 0;
    while (fgets(line_str, sizeof(line_str), dbfile)) {  // 修复这里缺少的括号
        if (strstr(line_str, tablename)) {
            found = 1;
            break;
        }
        fgetpos(dbfile, &pos_start);
    }

    if (!found) {
        printf("表不存在: %s\r\n", tablename);
        free(tablename);
        fclose(dbfile);
        return -1;
    }
    free(tablename);

    /* 定位删除起始点 */
    fsetpos(dbfile, &pos_start);
    long start_offset = ftell(dbfile);
    long end_offset = start_offset;

    /* 查找空行 */
    int empty_found = 0;
    while (fgets(line_str, sizeof(line_str), dbfile)) {
        /* 检查是否空行 */
        int is_empty = 1;
        for (char *p = line_str; *p && *p != '\n'; ++p) {
            if (!isspace(*p)) {
                is_empty = 0;
                break;
            }
        }
        if (is_empty) {
            end_offset = ftell(dbfile);
            empty_found = 1;
            break;
        }
    }

    /* 如果没有找到空行，截断到文件末尾 */
    if (!empty_found) {
        fseek(dbfile, 0, SEEK_END);
        end_offset = ftell(dbfile);
    }

    /* 移动文件内容 */
    long remaining = end_offset - start_offset;
    if (remaining > 0) {
        char buf[4096];
        size_t bytes;
        long write_pos = start_offset;

        fseek(dbfile, end_offset, SEEK_SET);
        while ((bytes = fread(buf, 1, sizeof(buf), dbfile)) > 0) {
            fseek(dbfile, write_pos, SEEK_SET);
            fwrite(buf, 1, bytes, dbfile);
            write_pos += bytes;
            fseek(dbfile, end_offset + write_pos - start_offset, SEEK_SET);
        }
    }

    /* 截断文件 */
    int fd = fileno(dbfile);
    ftruncate(fd, start_offset + (ftell(dbfile) - end_offset));

    fclose(dbfile);
    return 0;
}

/**
  * @brief  删除索引
  * @param 	dbname: 数据库名,cmd: 用户输入命令
  * @retval 0:成功
  */
int Delete_Index(char *dbname, char *cmd){

    return 0;
}
