﻿/********************************************************************
  Copyright (c) 2025 此乃刘同学.www.liustu.com.cn
  FileName : index.c
  Author   : 刘斌
  Version  : V1.0
  Date     : 2025-3-31
  Note     : C仿写sqlite3-嵌入式轻量级数据库
  History  : 2025-4-2 	V1.0 		完成初步编辑
********************************************************************/
#include "index.h"

/**
  * @brief  创建索引
  * @param 	dbname: 数据库名,cmd: 用户输入命令
  * @retval 0:成功
  */
int Create_Index(char *dbname, char *cmd) {
    char line_str[256];
    int line_flag = 0;
    fpos_t pos_start, pos_end;

    /* 检查结束符 */
    if(strchr(cmd, ';') == NULL) {
        printf("错误：缺少\";\"结束符\r\n");
        return -1;
    }

    /* 提取索引名 */
    char *index_start = strstr(cmd, "INDEX ");
    if(!index_start) {
        printf("语法错误：缺少INDEX关键字\r\n");
        return -1;
    }
    index_start += strlen("INDEX ");
    char *index_end = strchr(index_start, ' ');
    
    size_t index_len = index_end - index_start;
    char *indexname = malloc(index_len + 1);
    if(!indexname) {
        printf("内存分配失败\r\n");
        return -1;
    }
    strncpy(indexname, index_start, index_len);
    indexname[index_len] = '\0';
    printf("indexname --%s---\r\n",indexname );

    /* 提取表名 */
    char *table_start = strstr(cmd, "ON ");
    if(!table_start) {
        printf("语法错误：缺少TABLE关键字\r\n");
        return -1;
    }
    table_start += strlen("ON ");
    char *table_end = strchr(table_start, ' ');
    
    size_t table_len = table_end - table_start;
    char *tablename = malloc(table_len + 1);
    if(!tablename) {
        printf("内存分配失败\r\n");
        return -1;
    }
    strncpy(tablename, table_start, table_len);
    tablename[table_len] = '\0';
    printf("tablename --%s---\r\n",tablename );
    
    /* 提取属性 */
    char *key_start = strstr(cmd, "(");
    if(!key_start) {
        printf("语法错误：缺少属性关键字\r\n");
        return -1;
    }
    key_start += strlen("(");
    char *key_end = strchr(key_start, ')');
    
    size_t key_len = key_end - key_start;
    char *keyname = malloc(key_len + 1);
    if(!keyname) {
        printf("内存分配失败\r\n");
        return -1;
    }
    strncpy(keyname, key_start, key_len);
    keyname[key_len] = '\0';
    printf("keyname --%s---\r\n",keyname );

    /* 跳转到数据库存放的路径下 */
    chdir("sqlfile");

    /* 打开或创建数据库 */
    FILE *dbfile = fopen(dbname, "r+");
    
    /* 逐行读取 */
    while(fgets(line_str, sizeof(line_str), dbfile)){
        /* 当出现 tablename时,数据库中有相同的表 */
        if(strstr(line_str, tablename)) {
            // printf("出现相同表\n");
            line_flag = 1;
            // 保存当前位置
            fgetpos(dbfile, &pos_start);
            //free(paramter_cmd);
            break;
        }
    }
    
    /* 如果没有这个表，则就无法显示数据 */
    if(line_flag == 0){
        fclose(dbfile);
        dbfile = NULL;
        goto line_flag;
    }
    
    /* 提取出每个数据的数据名并显示 */
    char *line_parameter[256]; 
    int i = 0; 
    /* 读取表中数据内容 */
    while(fgets(line_str, sizeof(line_str), dbfile)){
        char *line_temp = (char *)malloc(strlen(line_str) + 1);
        if(!line_temp) {
            printf("注意：内存分配失败\r\n");
        }

        strcpy(line_temp, line_str);
        /* 去掉换行符 */
        line_str[strcspn(line_str, "\n")] = '\0';
        /* 读到END就结束了 */
        if(strcmp(line_str, "END") == 0){
            // 保存当前位置
            fgetpos(dbfile, &pos_end);
            break;
        }   

        char *token = strtok(line_str, " ");
        trim(token);
        if(token != NULL){
            line_parameter[i] = strdup(token);
        }
        // printf("--%s---\t", line_parameter[i]);
        /* 这个参数不能为空 */
        i ++;
        free(line_temp);
    }

    /* 获取key的位置 */
    int key_space, key_flag = 0;
    for (int j = 0; j < i; j++) {
        if(strcmp(keyname, line_parameter[j]) == 0) {
            key_space = i;
            key_flag = 1;
            break;
        }
    }
    if(key_flag == 0) {
        printf("错误: 当前属性不存在\r\n");
        goto key_falg;
    }
    printf("key_space = %d\r\n", key_space);

    /* 创建索引文件 */
    char *indexname_temp = indexname;
    char *file_fix = ".db";

    /* 构建文件名.db */
    strcat(indexname_temp, file_fix);
    printf("%s\n", indexname_temp);

    FILE *indexfile = fopen(indexname_temp, "a");

    /* 读取key的所有值存放到索引中 */
    fsetpos(dbfile, &pos_end);
    int value_count; 
    int index_space = 0;;
    /* 读取表中数据内容 */
    while (fgets(line_str, sizeof(line_str), dbfile) != NULL) {
        if (strcmp(line_str, "\n") == 0) {
            // 检测到空行，保存当前位置
            break;
        }

        /* 数据处理 */
        value_count = StrChrShow(line_str, '\\');
        char **vale_data = (char **)malloc(value_count * sizeof(char *));
        if(!vale_data) {
            printf("注意：内存分配失败\r\n");
        }
        char *value_token = strtok(line_str, "\\");
        for(int x = 0; x < value_count; x++){
            if(value_token == NULL)
                break;
            vale_data[x] = (char *)malloc((strlen(value_token) + 1) * sizeof(char));
            if(!vale_data[x]) {
                printf("注意：内存分配失败\r\n");
            }

            trim(value_token);
            strcpy(vale_data[x], value_token);

            value_token = strtok(NULL, "\\"); 
        }
        /* 显示 */
        char *index_data = (char *)malloc(strlen(vale_data[key_space-1])+6);
        if(!index_data) {
            printf("注意：内存分配失败\r\n");
        }
        sprintf(index_data, "%s:%d", vale_data[key_space-1], index_space);

        // 将index_data写入indexfile文件
        if (indexfile != NULL) {
            fprintf(indexfile, "%s\n", index_data);
        }

        index_space++;

        free(index_data);
        for(int i = 0; i < value_count; i++){
            free(vale_data[i]);
        }
        free(vale_data);
    }

    /* 关闭文件 */
    fclose(indexfile);
    indexfile = NULL;

key_falg:
    /* 释放 line_parameter 的内存 */
    for (int j = 0; j < i; j++) {
        free(line_parameter[j]);  // 释放每个元素的内存
    }

line_flag:
    /* 释放空间 */
    free(keyname);
    free(tablename);
    free(indexname);

    /* 关闭文件 */
    fclose(dbfile);
    dbfile = NULL;

    return 0;
}

/**
  * @brief  检测是否存在索引
  * @param 	dbname: 数据库名,tablename: 表名,field_names: 索引头
  * @retval 0:成功
  */
int Exist_index(char *dbname, char *tablename, char *field_names){

    return 0;
}