﻿/********************************************************************
  Copyright (c) 2025 此乃刘同学.www.liustu.com.cn
  FileName : mian.c
  Author   : 刘斌
  Version  : V1.0
  Date     : 2025-3-31
  Note     : C仿写sqlite3-嵌入式轻量级数据库
  History  : 2025-4-2 	V1.0 		完成初步编辑
********************************************************************/
#include "main.h"

/* 输入命令最大范围 */
#define CMD_LEN_MAX 256

int select_width = 20;

int main() {
    /* 用于缓存命令 */
    char cmd[CMD_LEN_MAX]; 
    /* 用于缓存数据库名称 */
    char db_name[50]; 
    /* 用于缓存表的名称 */
    char table_name[50];
    char start[50] = "liusql> ";
    printf("\033[1;35m"); // 加粗 + 青色文本
    printf("刘同学数据库 liustu.com.cn\n");

    printf("\033[1;33m"); // 加粗 + 黄色文本
    printf("输入\".help\"用于列出提示\n");

    printf("\033[1;32m"); // 加粗 + 绿色文本
    printf("使用\".open 库名\"打开数据库\n");

    printf("\033[0m"); // 重置所有样式

    while (1) {
        /* 输入命令 */
        printf("%s",start);
        fgets(cmd, sizeof(cmd), stdin);
        cmd[strcspn(cmd, "\n")] = '\0';

        if (strcmp(cmd, ".help") == 0) {                        // 输入.help显示所有提示
            show_all_help();
        } else if (strcmp(cmd, ".exit") == 0) {                 // 输入.exit退出整个功能
            break;
        } else if (strncmp(cmd, ".open ", 6) == 0) {            // 创建/打开数据库
            /* 提取数据库名称 */
            if (sscanf(cmd, ".open %s", db_name) == 1) {
                if( Open_DataBase(db_name) == 0){
                    sprintf(start, "liusql_%s> ",db_name);
                }
            } else {
                printf("错误：未指定数据库名称。\n");
            }
        } else if (strncmp(cmd, ".width ", 7) == 0) {                 // 添加元素
            char *width_start_ptr = strstr(cmd, ".width ") + 7;
            char *width_str = malloc(sizeof(char) * 100);
            if(!width_str ) {
                printf("注意：内存分配失败\r\n");
            }
            strncpy(width_str, width_start_ptr, 99); // 复制最多 99 个字符
            width_str[99] = '\0'; // 确保字符串以 \0 结尾
            select_width = atoi(width_str);
            printf("%d\n", select_width);
            free(width_str);
        } else if (strncmp(cmd, "CREATE TABLE ", 13) == 0) {                 // 创建表
            /* 当还没有打开数据库时，不能创建表 */
            if(strcmp(start, "liusql> ") == 0)
                continue;
            /* 提取创建的表名 */
            if(sscanf(cmd, "CREATE TABLE %[^(]", table_name) == 1){
                Create_Table(db_name, table_name);
            }
        } else if (strncmp(cmd, "INSERT INTO ", 12) == 0) {                 // 添加元素
            /* 当还没有打开数据库时，不能创建表 */
            if(strcmp(start, "liusql> ") == 0)
                continue;
            /* 提取创建的表名 */
            if(sscanf(cmd, "INSERT INTO %[^(]", table_name) == 1){
                Insert_Value(db_name, table_name, cmd);
            }
        } else if (strncmp(cmd, "SELECT ", 7) == 0) {  // 查询元素
            /* 当还没有打开数据库时，不能查询 */
            if (strcmp(start, "liusql> ") == 0) {
                continue;
            }
        
            /* 提取表名 */
            char *form_key = strstr(cmd, "FROM ");
            if (form_key == NULL) {
                printf("命令不完整，缺少 FROM\n");
                continue;
            }
            form_key = form_key + 5;  // 跳过 "FROM "
            char *cmd_end = strchr(form_key, ' ');
            if (cmd_end == NULL)
                cmd_end = strchr(form_key, ';');
            if (cmd_end == NULL) {
                printf("缺少结束符 ;\n");
                continue;
            }
            size_t table_len = cmd_end - form_key;
            strncpy(table_name, form_key, table_len);  // 使用 strncpy 复制表名
            table_name[table_len] = '\0';  // 添加字符串结束符
        
            /* 调用查询函数 */
            select_parameter(db_name, table_name, cmd); 
        } else if (strncmp(cmd, "UPDATE ", 7) == 0) {                 // 修改元素表
            /* 当还没有打开数据库时，不能创建表 */
            if(strcmp(start, "liusql> ") == 0)
                continue;
            /* 进入update函数 */
            cmd[strcspn(cmd, "\n")] = ' ';
            updata_parameter(db_name, cmd);
        } else if (strncmp(cmd, "DROP TABLE ", 10) == 0) {                 // 删除表
            /* 当还没有打开数据库时，不能创建表 */
            if(strcmp(start, "liusql> ") == 0)
                continue;
            /* 进入update函数 */
            cmd[strcspn(cmd, "\n")] = ' ';
            Delete_Table(db_name, cmd);
        } else if (strncmp(cmd, "CREATE INDEX ", 13) == 0) {                 // 删除表
            /* 当还没有打开数据库时，不能创建表 */
            if(strcmp(start, "liusql> ") == 0)
                continue;
            /* 进入update函数 */
            cmd[strcspn(cmd, "\n")] = ' ';
            Create_Index(db_name, cmd);
        } else {
            printf("错误：未知命令: %s\n", cmd);
        }
    }

    return 0;
}