﻿/********************************************************************
  Copyright (c) 2025 此乃刘同学.www.liustu.com.cn
  FileName : select.c
  Author   : 刘斌
  Version  : V1.0
  Date     : 2025-3-31
  Note     : C仿写sqlite3-嵌入式轻量级数据库
  History  : 2025-4-2 	V1.0 		完成初步编辑
********************************************************************/
#include "select.h"

#define MAX_CONDITIONS 10 // 假设最多有 10 个条件
#define PARAMETER_NUM 6

char parameter_sys[PARAMETER_NUM][5] = {">=", "<=", ">", "<", "==", "!="};

/**
  * @brief  处理WHERE命令
  * @param 	dbname: 数据库名,tablename: 表名,cmd: 用户命令,pos_start: 表在数据库中的起始位置
  * @retval 0：成功
  */
int select_parameter_where(char *dbname, char *tablename, char *cmd, fpos_t pos_start) {
    char line_str[256];
    fpos_t pos_end;

    /* 保存参数的字符串 */
    char *start = strstr(cmd, "SELECT ");
    start += 7;
    char *end = strstr(cmd, " FROM");
    size_t parameter_cmd_len = end - start;
    char *paramter_cmd = (char *)malloc(sizeof(char) * (parameter_cmd_len + 1));
    if(!paramter_cmd ) {
        printf("注意：内存分配失败\r\n");
    }
    strncpy(paramter_cmd, start, parameter_cmd_len);
    paramter_cmd[parameter_cmd_len] = '\0';
    if(paramter_cmd == NULL){
        printf("错误：WHERE后没有运算指令\n");
        free(paramter_cmd);
        return 1;
    }
    paramter_cmd[parameter_cmd_len] = '\0';
    

    /* 保存WHERE的字符串 */
    start = strstr(cmd, "WHERE ");
    start += 6;
    end = strstr(cmd, ";");
    size_t where_cmd_len = end - start;
    char *where_cmd = (char *)malloc(sizeof(char) * (where_cmd_len + 1));
    if(!where_cmd ) {
        printf("注意：内存分配失败\r\n");
    }
    strncpy(where_cmd, start, where_cmd_len);
    where_cmd[where_cmd_len] = '\0';
    // printf("WHERE 条件: %s\n", where_cmd);

    /* 存放条件的字段名、运算符和值 */
    char *field_names[MAX_CONDITIONS];
    char *operators[MAX_CONDITIONS];
    int field_count = 0; // 条件数量
    int operator_count = 0; // 运算符数量

    /* 查找逻辑运算符并分割字符串 */
    char *current_pos = where_cmd;
    while (1) {
        /* 是否有逻辑运算符 */
        char *and_operator = strstr(current_pos, " AND ");
        char *or_operator = strstr(current_pos, " OR ");
        char *between_operator = strstr(current_pos, " BETWEEN ");

        /* 出现的先后顺序 */
        char *operator_temp_point = Select_Strs_Less(and_operator, or_operator, between_operator);
        if (operator_temp_point == NULL) {
            /* 处理最后一个条件 */
            size_t condition_len = strlen(current_pos);
            field_names[field_count] = (char *)malloc((condition_len + 1) * sizeof(char));
            if(!field_names[field_count] ) {
                printf("注意：内存分配失败\r\n");
            }
            strncpy(field_names[field_count], current_pos, condition_len);
            field_names[field_count][condition_len] = '\0';
            field_count++;
            // printf("条件 %d: %s\n", field_count + 1, field_names[field_count]);
            break; // 退出循环
        }

        /* 计算逻辑运算符的长度 */
        size_t op_len = 0;
        if (strncmp(operator_temp_point, " AND ", 5) == 0) {
            op_len = 5; // " AND " 的长度
        } else if (strncmp(operator_temp_point, " OR ", 4) == 0) {
            op_len = 4; // " OR " 的长度
        } else if (strncmp(operator_temp_point, " BETWEEN ", 9) == 0) {
            op_len = 9; // " BETWEEN " 的长度
        }

        /* 存放逻辑运算符 */
        operators[operator_count] = (char *)malloc((op_len + 1) * sizeof(char));
        if(!operators[operator_count] ) {
            printf("注意：内存分配失败\r\n");
        }
        strncpy(operators[operator_count], operator_temp_point, op_len);
        operators[operator_count][op_len] = '\0';
        // printf("运算符 %d: %s\n", operator_count + 1, operators[operator_count]);
        operator_count++;

        /* 存放逻辑运算符之前的表达式 */
        size_t condition_len = operator_temp_point - current_pos;
        field_names[field_count] = (char *)malloc((condition_len + 1) * sizeof(char));
        if(!field_names[field_count] ) {
            printf("注意：内存分配失败\r\n");
        }
        strncpy(field_names[field_count], current_pos, condition_len);
        field_names[field_count][condition_len] = '\0';
        // printf("条件 %d: %s\n", field_count + 1, field_names[field_count]);
        field_count++;

        /* 更新字符指针 */
        current_pos = operator_temp_point + op_len; // 跳过逻辑运算符
    }

    /* 处理每个变量之间的表达式 */

    /* 打开或创建数据库 */
    FILE *dbfile = fopen(dbname, "r+");

    /* 跳转到表的行 */
    fsetpos(dbfile, &pos_start);

    char *line_parameter[256]; 
    int line_flag = 0;

    /* 读取表中数据内容 */
    for(int i = 0; i<=field_count ;i++) { // 判断是不是有索引
        if(Exist_index(dbname, tablename, field_names[field_count]) == 0){
            printf("存在索引%s\r\n", field_names[field_count]);
        }
    }
    while(fgets(line_str, sizeof(line_str), dbfile)) {
        char *line_temp = (char *)malloc(strlen(line_str) + 1);
        if(!line_temp ) {
            printf("注意：内存分配失败\r\n");
        }
        strcpy(line_temp, line_str);
        /* 去掉换行符 */
        line_str[strcspn(line_str, "\n")] = '\0';
        /* 读到END就结束了 */
        if(strcmp(line_str, "END") == 0) {
            // 保存当前位置
            fgetpos(dbfile, &pos_end);
            break;
        }   

        char *token = strtok(line_str, " ");
        trim(token);

        /* 存储变量 */
        if(token != NULL) {
            line_parameter[line_flag] = strdup(token);
        }
        // printf("---%s---", line_parameter[line_flag]);

        line_flag ++;
        free(line_temp);
    }

    /* 判断非法参数 */
    int field_point[field_count];
    memset(field_point, 0, field_count);
    for(int x = 0; x < field_count; x++){
        int line_sign = 1;
        for(int i = 0; i < line_flag; i++){
            if(strstr(field_names[x], line_parameter[i]) != NULL){
                field_point[x] = i;
                line_sign = 0;
                break;
            }
        }
        if(line_sign){
            printf("错误:处理的参数表中不存在\n");
            /* 关闭文件 */
            fclose(dbfile);
            dbfile = NULL;

            /* 释放空间 */
            free(paramter_cmd);
            free(where_cmd);
            for (int i = 0; i < field_count; i++) {
                free(field_names[i]);
            }
            for (int i = 0; i < operator_count; i++) {
                free(operators[i]);
            }
            for(int i = 0; i < line_flag; i++) {
                free(line_parameter[i]);
            }
            return 1;
        }
    }
    
    // /* 通过指针索引对应列表中的参数 */
    // for(int x = 0; x < field_count; x++){
    //     printf("----%s----", line_parameter[field_point[x]]);
    // }

    /* *全部显示 */
    int value_count;
    if(strstr(cmd ,"SELECT * FROM")){
        
        for (int j = 0;j < line_flag;j++){
            printf("%-*s",select_width , line_parameter[j]);
        }
        printf("\n");

        /* 显示后面所有内容 */
        fsetpos(dbfile, &pos_end);
        while (fgets(line_str, sizeof(line_str), dbfile) != NULL) {
            if (strcmp(line_str, "\n") == 0) {
                // 检测到空行，保存当前位置
                break;
            }
            /* 数据处理 */
            value_count = StrChrShow(line_str, '\\');
            char **vale_data = (char **)malloc(value_count * sizeof(char *));
            if(!vale_data ) {
                printf("注意：内存分配失败\r\n");
            }
            char *value_token = strtok(line_str, "\\");
            for(int x = 0; x < value_count; x++){
                if(value_token == NULL)
                    break;
                vale_data[x] = (char *)malloc((strlen(value_token) + 1) * sizeof(char));
                if(!vale_data[x] ) {
                    printf("注意：内存分配失败\r\n");
                }
                trim(value_token);
                strcpy(vale_data[x], value_token);
                value_token = strtok(NULL, "\\"); 
            }
            // printf("\n");

            /* 判断表达式 */

            /* 解析表达式 */
            char *field_name = (char *)malloc(sizeof(char) * 100);      // 字段名
            if(!field_name) {
                printf("注意：内存分配失败\r\n");
            }
            char *arithmetic_op = (char *)malloc(sizeof(char) * 3);   // 算术运算符
            if(!arithmetic_op) {
                printf("注意：内存分配失败\r\n");
            }
            char *compare_value = (char *)malloc(sizeof(char) * 100);   // 比较值
            if(!compare_value) {
                printf("注意：内存分配失败\r\n");
            }

            /* 没有逻辑运算符 */
            if (operator_count == 0) {
                /* 查找算术运算符 */
                for (int i = 0; i < PARAMETER_NUM; i++) {
                    char *op_pos = strstr(field_names[0], parameter_sys[i]);
                    if (op_pos != NULL) {
                        /* 提取字段名 */
                        size_t field_len = op_pos - field_names[0];
                        strncpy(field_name, field_names[0], field_len);
                        field_name[field_len] = '\0';
                        trim(field_name); // 去除比较值前后的空格

                        /* 提取算术运算符 */
                        strcpy(arithmetic_op, parameter_sys[i]);
                        trim(arithmetic_op);

                        /* 提取比较值 */
                        strcpy(compare_value, op_pos + strlen(parameter_sys[i]));
                        trim(compare_value); // 去除比较值前后的空格

                        /* 检测先后顺序 */
                        int field_flag = 1;
                        for(int x = 0; x < field_count; x++){
                            // printf("%s---",line_parameter[field_point[x]]);
                            if(strcmp(line_parameter[field_point[x]], compare_value) == 0){
                                field_flag = 0;
                                break;
                            }
                        }

                        if(!field_flag){  // 字段名在运算符的后面
                            if(strcmp(arithmetic_op, ">=") == 0)
                                strcpy(arithmetic_op, "<=");
                            else if(strcmp(arithmetic_op, "<=") == 0)
                                strcpy(arithmetic_op, ">=");
                            else if(strcmp(arithmetic_op, "<") == 0)
                                strcpy(arithmetic_op, ">");
                            else if(strcmp(arithmetic_op, ">") == 0)
                                strcpy(arithmetic_op, "<");

                            // 交换 field_name 和 compare_value
                            swap(&field_name, &compare_value);
                        }

                        // printf("字段名: %s\n", field_name);
                        // printf("算术运算符: %s\n", arithmetic_op);
                        // printf("比较值: %s\n", compare_value);
                        break;
                    }
                }

                /* 判断运算符 */
                trim(compare_value);
                trim(vale_data[field_point[0]]);
                // printf("---%s-----%s-----\n", vale_data[field_point[0]], compare_value);

                int print_flag = 0;
                float field_value = strtof(vale_data[field_point[0]], NULL);
                float compare_float = strtof(compare_value, NULL);

                /* 判断是否为浮点数比较 */
                if (field_value != 0.0f || compare_float != 0.0f || strcmp(vale_data[field_point[0]], "0") == 0 || strcmp(compare_value, "0") == 0) {
                    if(strcmp(arithmetic_op, ">=") == 0){
                        if(field_value >= compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "<=") == 0){
                        if(field_value <= compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "!=") == 0){
                        if(field_value != compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "==") == 0){
                        if(field_value == compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, ">") == 0){
                        if(field_value > compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "<") == 0){
                        if(field_value < compare_float)
                            print_flag = 1;
                    }
                } else {
                    /* 字符串比较 */
                    int cmp_result = strcmp(vale_data[field_point[0]], compare_value);
                    if(strcmp(arithmetic_op, "!=") == 0){
                        if(cmp_result != 0)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "==") == 0){
                        if(cmp_result == 0)
                            print_flag = 1;
                    }
                }
                
                if(print_flag){
                    for(int x = 0; x < value_count; x++){
                        printf("%-*s",select_width, vale_data[x]);
                    }
                    printf("\n");
                }

            }
            /* 带有逻辑运算符 */
            else {
                int prev_result = 0;
                int is_between = 0;
                int cond_idx;
                // 遍历所有条件
                for (cond_idx = 0; cond_idx < field_count; cond_idx++) {
                    // 解析单个条件
                    char *field_name = malloc(100 * sizeof(char));
                    if(!field_name) {
                        printf("注意：内存分配失败\r\n");
                    }
                    char *arithmetic_op = malloc(3 * sizeof(char));
                    if(!arithmetic_op) {
                        printf("注意：内存分配失败\r\n");
                    }
                    char *compare_value = malloc(100 * sizeof(char));
                    if(!compare_value) {
                        printf("注意：内存分配失败\r\n");
                    }
                    memset(field_name, 0, 100);
                    memset(arithmetic_op, 0, 3);
                    memset(compare_value, 0, 100);

                    // 解析条件表达式（类似无逻辑运算符的处理）
                    for (int i = 0; i < PARAMETER_NUM; i++) {
                        char *op_pos = strstr(field_names[cond_idx], parameter_sys[i]);
                        if (op_pos) {
                            // 提取字段名
                            size_t field_len = op_pos - field_names[cond_idx];
                            strncpy(field_name, field_names[cond_idx], field_len);
                            trim(field_name);

                            // 提取运算符
                            strcpy(arithmetic_op, parameter_sys[i]);
                            trim(arithmetic_op);

                            // 提取比较值
                            strcpy(compare_value, op_pos + strlen(parameter_sys[i]));
                            trim(compare_value);

                            // 检测字段名和比较值顺序
                            int field_flag = 1;
                            for (int x = 0; x < field_count; x++) {
                                if (strcmp(line_parameter[field_point[x]], compare_value) == 0) {
                                    field_flag = 0;
                                    break;
                                }
                            }

                            if (!field_flag) {
                                // 交换运算符方向
                                if (strcmp(arithmetic_op, ">=") == 0) strcpy(arithmetic_op, "<=");
                                else if (strcmp(arithmetic_op, "<=") == 0) strcpy(arithmetic_op, ">=");
                                else if (strcmp(arithmetic_op, "<") == 0) strcpy(arithmetic_op, ">");
                                else if (strcmp(arithmetic_op, ">") == 0) strcpy(arithmetic_op, "<");
                                swap(&field_name, &compare_value);
                            }
                            break;
                        }
                    }
                }

                // 获取当前字段的值
                char *current_field_value = vale_data[field_point[cond_idx]];
                trim(current_field_value);
                float field_float = strtof(current_field_value, NULL);
                float compare_float = strtof(compare_value, NULL);

                // 判断是否为浮点数比较
                int is_float_comparison = (field_float != 0.0f || compare_float != 0.0f || 
                                        strcmp(current_field_value, "0") == 0 || 
                                        strcmp(compare_value, "0") == 0);

                // 判断当前条件是否满足
                int cond_met = 0;
                if (is_float_comparison) {
                    if (strcmp(arithmetic_op, ">=") == 0) {
                        cond_met = (field_float >= compare_float);
                    } else if (strcmp(arithmetic_op, "<=") == 0) {
                        cond_met = (field_float <= compare_float);
                    } else if (strcmp(arithmetic_op, "!=") == 0) {
                        cond_met = (field_float != compare_float);
                    } else if (strcmp(arithmetic_op, "==") == 0) {
                        cond_met = (field_float == compare_float);
                    } else if (strcmp(arithmetic_op, ">") == 0) {
                        cond_met = (field_float > compare_float);
                    } else if (strcmp(arithmetic_op, "<") == 0) {
                        cond_met = (field_float < compare_float);
                    }
                } else {
                    int cmp_result = strcmp(current_field_value, compare_value);
                    if (strcmp(arithmetic_op, "!=") == 0) {
                        cond_met = (cmp_result != 0);
                    } else if (strcmp(arithmetic_op, "==") == 0) {
                        cond_met = (cmp_result == 0);
                    }
                }

                // 处理逻辑运算符
                if (cond_idx == 0) {
                    prev_result = cond_met;
                } else {
                    char *current_op = operators[cond_idx - 1];  // 运算符数组比条件少一个
                    if (strstr(current_op, "AND")) {
                        prev_result = prev_result && cond_met;
                    } else if (strstr(current_op, "OR")) {
                        prev_result = prev_result || cond_met;
                    } else if (strstr(current_op, "BETWEEN")) {
                        // BETWEEN特殊处理：需要解析两个值
                        char *between_start = strtok(compare_value, " AND ");
                        char *between_end = strtok(NULL, " AND ");
                        if (!between_start || !between_end) {
                            printf("错误：BETWEEN 格式不正确\n");
                            break;
                        }
                        float start_val = strtof(between_start, NULL);
                        float end_val = strtof(between_end, NULL);
                        prev_result = (field_float >= start_val) && (field_float <= end_val);
                        is_between = 1;  // 标记BETWEEN处理
                    }
                }

                // 释放临时变量
                free(field_name);
                free(arithmetic_op);
                if (!is_between) free(compare_value);  // BETWEEN时compare_value已被strtok修改

                // 最终判断
                if (prev_result) {
                    for (int x = 0; x < value_count; x++) {
                        printf("%-*s", select_width, vale_data[x]);
                    }
                    printf("\n");
                }
            }

            /* 释放内存 */
            free(field_name);
            free(arithmetic_op);
            free(compare_value);
            
        // FAILED:
            for(int i = 0; i < value_count; i++){
                free(vale_data[i]);
            }
            free(vale_data);

        }
    }
    else {
        /* 获取每个参数的内容 */
        int parameter_count = StrChrShow(paramter_cmd, ',') + 1;
        char **parameter_value = (char **)malloc(parameter_count * sizeof(char *));
        if(!parameter_value) {
            printf("注意：内存分配失败\r\n");
        }
        char *token = strtok(paramter_cmd, ",");
        for(int x = 0; x < parameter_count; x++){
            if(token == NULL)
                break;
            parameter_value[x] = (char *)malloc((strlen(token) + 1) * sizeof(char));
            if(!parameter_value[x]) {
                printf("注意：内存分配失败\r\n");
            }
            if(!parameter_value[x]) {
                printf("注意：内存分配失败\r\n");
            }
            strcpy(parameter_value[x], token);
            token = strtok(NULL, ",");
        }
    
        /* 判断输入的参数，表中有 */
        if (parameter_count > line_flag) {
            printf("错误：存在了表中没有的元素\n");
        }
    
        /* 每个参数的数组位置 */
        int parameter_line[parameter_count];
        memset(parameter_line, 0, sizeof(parameter_line));
    
        for (int y = 0; y < parameter_count; y++) {
            int param_found = 0;  // 标记参数是否在表中找到
            for (int x = 0; x < line_flag; x++) {
                trim(parameter_value[y]);
                trim(line_parameter[x]);
                if (strcmp(line_parameter[x], parameter_value[y]) == 0) {
                    param_found = 1;
                    parameter_line[y] = x;
                    break;
                }
            }
            if (!param_found) {
                printf("错误：参数 '%s' 不在表中\n", parameter_value[y]);
            }
        }
    
        /* 显示开头 */
        for (int y = 0; y < parameter_count; y++) {
            printf("%-*s", select_width, parameter_value[y]);
        }
        printf("\n");
    
        /* 显示后面所有内容 */
        fsetpos(dbfile, &pos_end);
        while (fgets(line_str, sizeof(line_str), dbfile) != NULL) {
            if (strcmp(line_str, "\n") == 0) {
                // 检测到空行，保存当前位置
                break;
            }
            /* 数据处理 */
            value_count = StrChrShow(line_str, '\\');
            char **vale_data = (char **)malloc(value_count * sizeof(char *));
            if(!vale_data) {
                printf("注意：内存分配失败\r\n");
            }
            char *value_token = strtok(line_str, "\\");
            for(int x = 0; x < value_count; x++){
                if(value_token == NULL)
                    break;
                vale_data[x] = (char *)malloc((strlen(value_token) + 1) * sizeof(char));
                if(!vale_data[x]) {
                    printf("注意：内存分配失败\r\n");
                }
                trim(value_token);
                strcpy(vale_data[x], value_token);
                value_token = strtok(NULL, "\\"); 
            }
    
            /* 判断表达式 */
            char *field_name = (char *)malloc(sizeof(char) * 100);      // 字段名
            if(!field_name) {
                printf("注意：内存分配失败\r\n");
            }
            char *arithmetic_op = (char *)malloc(sizeof(char) * 3);   // 算术运算符
            if(!arithmetic_op) {
                printf("注意：内存分配失败\r\n");
            }
            char *compare_value = (char *)malloc(sizeof(char) * 100);   // 比较值
            if(!compare_value) {
                printf("注意：内存分配失败\r\n");
            }
            
            /* 没有逻辑运算符 */
            if (operator_count == 0) {
                for (int j = 0; j < PARAMETER_NUM; j++) {
                    char *op_pos = strstr(field_names[0], parameter_sys[j]);
                    if (op_pos != NULL) {
                        /* 提取字段名 */
                        size_t field_len = op_pos - field_names[0];
                        field_name = (char *)malloc((field_len + 1) * sizeof(char));
                        if(!field_name) {
                            printf("注意：内存分配失败\r\n");
                        }
                        strncpy(field_name, field_names[0], field_len);
                        field_name[field_len] = '\0';
                        trim(field_name); // 去除比较值前后的空格
    
                        /* 提取算术运算符 */
                        strcpy(arithmetic_op, parameter_sys[j]);
                        trim(arithmetic_op);

                        /* 提取比较值 */
                        strcpy(compare_value, op_pos + strlen(parameter_sys[j]));
                        trim(compare_value); // 去除比较值前后的空格

                        /* 检测先后顺序 */
                        int field_flag = 1;
                        for(int x = 0; x < field_count; x++){
                            if(strcmp(line_parameter[field_point[x]], compare_value) == 0){
                                field_flag = 0;
                                break;
                            }
                        }
    
                        if(!field_flag){  // 字段名在运算符的后面
                            if(strcmp(arithmetic_op, ">=") == 0)
                                strcpy(arithmetic_op, "<=");
                            else if(strcmp(arithmetic_op, "<=") == 0)
                                strcpy(arithmetic_op, ">=");
                            else if(strcmp(arithmetic_op, "<") == 0)
                                strcpy(arithmetic_op, ">");
                            else if(strcmp(arithmetic_op, ">") == 0)
                                strcpy(arithmetic_op, "<");
    
                            // 交换 field_name 和 compare_value
                            swap(&field_name, &compare_value);
                        }
    
                        break;
                    }
                }
    
                /* 判断运算符 */
                trim(compare_value);
                trim(vale_data[field_point[0]]);
                    
                int print_flag = 0;
                float field_value = strtof(vale_data[field_point[0]], NULL);
                float compare_float = strtof(compare_value, NULL);

                if (field_value != 0.0f || compare_float != 0.0f || strcmp(vale_data[field_point[0]], "0") == 0 || strcmp(compare_value, "0") == 0) {
                    if(strcmp(arithmetic_op, ">=") == 0){
                        if(field_value >= compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "<=") == 0){
                        if(field_value <= compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "!=") == 0){
                        if(field_value != compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "==") == 0){
                        if(field_value == compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, ">") == 0){
                        if(field_value > compare_float)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "<") == 0){
                        if(field_value < compare_float)
                            print_flag = 1;
                    }
                } else {
                    /* 字符串比较 */
                    int cmp_result = strcmp(vale_data[field_point[0]], compare_value);
                    if(strcmp(arithmetic_op, "!=") == 0){
                        if(cmp_result != 0)
                            print_flag = 1;
                    }else if(strcmp(arithmetic_op, "==") == 0){
                        if(cmp_result == 0)
                            print_flag = 1;
                    }
                }

                if(print_flag){
                    for(int x = 0; x < parameter_count; x++){
                        printf("%-*s",select_width, vale_data[ parameter_line[x]]);
                    }
                    printf("\n");
                }
            }
            /* 带有逻辑运算符 */
            else {
                ;
            }
            
            /* 释放内存 */
            if (field_name != NULL) free(field_name);
            if (arithmetic_op != NULL) free(arithmetic_op);
            if (compare_value != NULL) free(compare_value);
    
            for(int i = 0; i < value_count; i++){
                free(vale_data[i]);
            }
            free(vale_data);
    
        }
    
        /* 释放内存 */
        for (int x = 0; x < parameter_count; x++) {
            free(parameter_value[x]); 
        }
        free(parameter_value);
    }

    /* 跳转到数据库存放的路径下 */
    chdir("sqlfile");

    /* 关闭文件 */
    fclose(dbfile);
    dbfile = NULL;

    /* 释放空间 */
    free(paramter_cmd);
    free(where_cmd);
    for (int i = 0; i < field_count; i++) {
        free(field_names[i]);
    }
    for (int i = 0; i < operator_count; i++) {
        free(operators[i]);
    }
    for(int i = 0; i < line_flag; i++) {
        free(line_parameter[i]);
    }

    return 0;
}

/**
  * @brief  查询命令
  * @param 	dbname: 数据库名,tablename: 表名,cmd: 用户命令
  * @retval 0：成功
  */
int select_parameter(char *dbname, char *tablename, char *cmd){
    char line_str[256];
    int line_flag = 0;
    fpos_t pos_start;
    fpos_t pos_end;
    int parameter_count;

    /* 保存参数的字符串 */
    char *start = strstr(cmd, "SELECT ");
    start += 7;
    char *end = strstr(cmd, " FROM");
    size_t parameter_cmd_len = end - start;
    char *paramter_cmd = (char *)malloc(sizeof(char) * parameter_cmd_len);
    if(!paramter_cmd) {
        printf("注意：内存分配失败\r\n");
    }
    strncpy(paramter_cmd, start, parameter_cmd_len);
    paramter_cmd[parameter_cmd_len] = '\0';
    // printf("%s\n", paramter_cmd);
    
    /* 跳转到数据库存放的路径下 */
    chdir("sqlfile");

    /* 打开或创建数据库 */
    FILE *dbfile = fopen(dbname, "r+");

    /* 逐行读取 */
    while(fgets(line_str, sizeof(line_str), dbfile)){
        /* 当出现 tablename时,数据库中有相同的表 */
        if(strstr(line_str, tablename)) {
            // printf("出现相同表\n");
            line_flag = 1;
            // 保存当前位置
            fgetpos(dbfile, &pos_start);
            //free(paramter_cmd);
            break;
        }
    }

    /* 如果没有这个表，则就无法显示数据 */
    if(line_flag == 0){
        fclose(dbfile);
        dbfile = NULL;
        free(paramter_cmd);
        return 1;
    }

    /* 如果存在WHERE关键字 */
    if(strstr(cmd ,"WHERE ") != NULL){
        /* 筛选出条件 */
        // char *where_clause = strstr(cmd, "WHERE ") + 6;  // 跳过 "WHERE "
        // printf("WHERE 条件: %s\n", where_clause);

        /* 进入带有WHERE的处理部分 */
        select_parameter_where(dbname, tablename, cmd, pos_start);

        /* 关闭文件 */
        fclose(dbfile);
        dbfile = NULL;
        return 0;
    }

    /* 不带WHERE 正常判断 */
    
    /* 提取出每个数据的数据名并显示 */
    char *line_parameter[256]; 
    int i = 0; 
    /* 读取表中数据内容 */
    while(fgets(line_str, sizeof(line_str), dbfile)){
        char *line_temp = (char *)malloc(strlen(line_str) + 1);
        if(!line_temp) {
            printf("注意：内存分配失败\r\n");
        }
        strcpy(line_temp, line_str);
        /* 去掉换行符 */
        line_str[strcspn(line_str, "\n")] = '\0';
        /* 读到END就结束了 */
        if(strcmp(line_str, "END") == 0){
            // 保存当前位置
            fgetpos(dbfile, &pos_end);
            break;
        }   

        char *token = strtok(line_str, " ");
        trim(token);
        if(token != NULL){
            line_parameter[i] = strdup(token);
        }
        // printf("%s\t", line_parameter[i]);
        // printf("%s\n", line_type[i]);
        //  printf("%s \n",line_temp);
        /* 这个参数不能为空 */
        i ++;
        free(line_temp);
    }
    
    int value_count;
    /* *全部显示 */
    if(strstr(cmd ,"SELECT * FROM")){
        
        /* 释放空间 */
        for (int j = 0;j < i;j++){
            printf("%-*s",select_width , line_parameter[j]);
        }
        printf("\n");

        /* 显示后面所有内容 */
        fsetpos(dbfile, &pos_end);
        while (fgets(line_str, sizeof(line_str), dbfile) != NULL) {
            if (strcmp(line_str, "\n") == 0) {
                // 检测到空行，保存当前位置
                break;
            }
            /* 数据处理 */
            value_count = StrChrShow(line_str, '\\');
            char **vale_data = (char **)malloc(value_count * sizeof(char *));
            if(!vale_data) {
                printf("注意：内存分配失败\r\n");
            }
            char *value_token = strtok(line_str, "\\");
            for(int x = 0; x < value_count; x++){
                if(value_token == NULL)
                    break;
                vale_data[x] = (char *)malloc((strlen(value_token) + 1) * sizeof(char));
                if(!vale_data[x]) {
                    printf("注意：内存分配失败\r\n");
                }
                trim(value_token);
                strcpy(vale_data[x], value_token);

                printf("%-*s",select_width, vale_data[x]);
                value_token = strtok(NULL, "\\"); 
            }
            printf("\n");

            for(int i = 0; i < value_count; i++){
                free(vale_data[i]);
            }
            free(vale_data);
        }

    }
    /* 并不输出所有数据,根据输入的参数显示 */
    else{
        /* 获取每个参数的内容 */
        parameter_count = StrChrShow(paramter_cmd, ',') + 1;;
        char **parameter_value = (char **)malloc(parameter_count * sizeof(char *));
        if(!parameter_value) {
            printf("注意：内存分配失败\r\n");
        }
        char *token = strtok(paramter_cmd, ",");
        for(int x = 0; x < parameter_count; x++){
            if(token == NULL)
                break;
            parameter_value[x] = (char *)malloc((strlen(token) + 1) * sizeof(char));
            if(!parameter_value[x]) {
                printf("注意：内存分配失败\r\n");
            }
            strcpy(parameter_value[x], token);
            token = strtok(NULL, ",");
        }
        
        /* 判断输入的参数，表中有 */
        if (parameter_count > i) {
            printf("错误：存在了表中没有的元素\n");
        }

        /* 每个参数的数组位置 */
        int parameter_line[parameter_count];
        memset(parameter_line, 0, sizeof(parameter_line));

        for (int y = 0; y < parameter_count; y++) {
            int param_found = 0;  // 标记参数是否在表中找到
            for (int x = 0; x < i; x++) {
                trim(parameter_value[y]);
                trim(line_parameter[x]);
                if (strcmp(line_parameter[x], parameter_value[y]) == 0) {
                    param_found = 1;
                    parameter_line[y] = x;
                    break;
                }
            }
            if (!param_found) {
                printf("错误：参数 '%s' 不在表中\n", parameter_value[y]);
            }
        }
        
        /* 显示开头 */
        for (int y = 0; y < parameter_count; y++) {
            printf("%-*s", select_width, parameter_value[y]);
        }
        printf("\n");

        // for(int i=0; i < parameter_count; i++){
        //     printf("%s ", line_parameter[parameter_line[i]]);
        // }

        /* 显示后面所有内容 */
        fsetpos(dbfile, &pos_end);
        while (fgets(line_str, sizeof(line_str), dbfile) != NULL) {
            /* 数据处理 */
            value_count = StrChrShow(line_str, '\\');
            char **vale_data = (char **)malloc(value_count * sizeof(char *));
            if(!vale_data) {
                printf("注意：内存分配失败\r\n");
            }
            char *value_token = strtok(line_str, "\\");
            for(int x = 0; x < value_count; x++){
                if(value_token == NULL)
                    break;
                vale_data[x] = (char *)malloc((strlen(value_token) + 1) * sizeof(char));
                if(!vale_data[x]) {
                    printf("注意：内存分配失败\r\n");
                }
                trim(value_token);
                strcpy(vale_data[x], value_token);

                value_token = strtok(NULL, "\\"); 
            }
            /* 显示 */
            if (strcmp(line_str, "\n") != 0){
                for(int y = 0; y < parameter_count; y++){
                    
                    printf("%-*s",select_width, vale_data[parameter_line[y]]);
                }
                printf("\n");
            }

            for(int i = 0; i < value_count; i++){
                free(vale_data[i]);
            }
            free(vale_data);

            if (strcmp(line_str, "\n") == 0) {
                // 检测到空行，保存当前位置
                break;
            }
        }

        /* 释放内存 */
        for (int x = 0; x < parameter_count; x++) {
            // printf("%s\t", parameter_value[x] );
            free(parameter_value[x]); 
        }
        free(parameter_value);
    }

    /* 释放 line_parameter 的内存 */
    for (int j = 0; j < i; j++) {
        free(line_parameter[j]);  // 释放每个元素的内存
    }

    /* 关闭文件 */
    fclose(dbfile);
    dbfile = NULL;

    /* 释放内存 */
    free(paramter_cmd);
    return 0;
}

/**
  * @brief  去除空格
  * @param 	去除空格的字符串
  * @retval NULL
  */
void trim(char *str) {
    char *start = str;
    char *end;

    /* 去除开头空格 */
    while (isspace((unsigned char)*start)) {
        start++;
    }

    /* 如果字符串全是空格 */
    if (*start == 0) {
        *str = 0;
        return;
    }

    /* 去除结尾的空格 */
    end = start + strlen(start) - 1;
    while (end > start && isspace((unsigned char)*end)) {
        end--;
    }
    *(end + 1) = '\0';

    memmove(str, start, end - start + 2);
}

/**
  * @brief  计算子字符串在字符串中出现的次数
  * @param 	str: 原字符串,sub: 子字符串
  * @retval 出现字数
  */
int Count_SubString(const char *str, const char *sub) {
    int count = 0;
    size_t sub_len = strlen(sub); // 子字符串的长度

    // 遍历字符串
    while((str = strstr(str, sub))) { // 查找子字符串
        count++; // 找到一次，计数加1
        str += sub_len; // 移动指针到子字符串之后
    }

    return count;
}

/**
  * @brief  判断字符指针的地址大小（最小值）
  * @param 	a,b,c: 对比的字符指针
  * @retval 最小的字符指针
  */
char *Select_Strs_Less(char *a, char *b, char *c) {
    /* 全空 */
    if (a == NULL && b == NULL && c == NULL)
        return NULL;

    /* 初始化最小指针 */
    char *min_ptr = NULL;

    /* 找到第一个非空指针 */
    if (a != NULL) {
        min_ptr = a;
    } else if (b != NULL) {
        min_ptr = b;
    } else if (c != NULL) {
        min_ptr = c;
    }

    /* 比较 a、b、c，找到最小的指针 */
    if (a != NULL && a < min_ptr) {
        min_ptr = a;
    }
    if (b != NULL && b < min_ptr) {
        min_ptr = b;
    }
    if (c != NULL && c < min_ptr) {
        min_ptr = c;
    }

    return min_ptr;
}

/**
  * @brief  字符串交换
  * @param 	str1,str2: 要交换的字符串指针
  * @retval NULL
  */
void swap(char **str1, char **str2) {
    char *swap_temp = NULL;
    swap_temp = *str1;
    *str1 = *str2;
    *str2 = swap_temp;
    // free(swap_temp);
}
