﻿/********************************************************************
  Copyright (c) 2025 此乃刘同学.www.liustu.com.cn
  FileName : updata.c
  Author   : 刘斌
  Version  : V1.0
  Date     : 2025-3-31
  Note     : C仿写sqlite3-嵌入式轻量级数据库
  History  : 2025-4-2 	V1.0 		完成初步编辑
********************************************************************/
#include "updata.h"

/**
  * @brief  update操作
  * @param 	dbname: 数据库名,cmd: 用户输入指令
  * @retval 0:成功
  */
int updata_parameter(char *dbname, char *cmd){
    int start_len = (strlen(dbname) + 7);
    char value_cmd[100]; 

    char line_str[512];     // 逐行读取缓存
    char final_cmd[512];    // 指令
    char tablename[100];    // 表名
    char *set_cmd;          // set指令
    fpos_t pos_start;       // 表的起始位置
    fpos_t pos_end;   
    int line_flag = 0;      // 表标志位
    strcpy(final_cmd, cmd);

    /* 读取完整数据 */
    while(1){
        if(strchr(final_cmd, ';')){
            break;
        }
        /* 读取命令 */
        int count = start_len;
        while (count--)
            printf(" ");
        printf("> ");
        fgets(value_cmd, sizeof(value_cmd), stdin);
        
        /* 去掉换行符 */
        value_cmd[strcspn(value_cmd, "\n")] = ' ';
        strcat(final_cmd, value_cmd);
    }

    /* 判断SET关键字 */
    if(strstr(final_cmd, "SET") == 0){
        printf("错误：缺少SET关键字\n");
        return 1;
    }

    /* 提取表名字 */
    const char *start = strstr(final_cmd, "UPDATE ");
    const char *end = strstr(final_cmd, " SET");
    if (start && end && start < end) {
        /* 计算子字符串A和B之间的内容 */
        start += strlen("UPDATE "); // 跳过子字符串A
        int length = end - start;    // 计算需要提取的长度

        strncpy(tablename, start, length);
        tablename[length] = '\0'; // 添加字符串结束符

    } else {
        printf("错误：缺少表名\n");
        return 1;
    }

    /* 跳转到数据库存放的路径下 */
    chdir("sqlfile");

    /* 打开或创建数据库 */
    FILE *dbfile = fopen(dbname, "r+");
    if (dbfile == NULL) {
        printf("错误：无法打开数据库文件\n");
        return 1;
    }

    /* 逐行读取 */
    while(fgets(line_str, sizeof(line_str), dbfile)){
        /* 当出现 tablename时,数据库中有相同的表 */
        if(strstr(line_str, tablename)) {
            line_flag = 1;
            // 保存当前位置
            fgetpos(dbfile, &pos_start);
            break;
        }
    }

    /* 如果没有这个表，则就无法插入数据 */
    if(line_flag == 0){
        printf("错误：数据库中没有该表\n");
        fclose(dbfile);
        return 1;
    }

    /* 提取WHERE条件 */
    char *where_clause = strstr(final_cmd, " WHERE ");
    char *where_clause_temp = where_clause;
    char where_field[100] = {0};
    char where_value[100] = {0};
    int where_field_index = -1;
    if (where_clause) {
        where_clause += strlen(" WHERE ");
        sscanf(where_clause, "%[^ =] = %s", where_field, where_value);
        // 去除分号
        char *semicolon = strchr(where_value, ';');
        if (semicolon) {
            *semicolon = '\0';
        }
    }
    printf("WHERE %s\r\n" , where_clause ? where_clause : "");

    /* 提取SET命令 */
    const char *set_start = strstr(final_cmd, "SET ");
    const char *set_end;
    if (where_clause_temp) {
        set_end = where_clause_temp - 1; // 从 WHERE 前一个字符结束
    } else {
        set_end = strchr(final_cmd, ';');
    }
    if (set_start && set_end && set_start < set_end) {
        /* 计算子字符串A和B之间的内容 */
        set_start += strlen("SET "); // 跳过子字符串A
        int length = set_end - set_start;    // 计算需要提取的长度

        set_cmd = (char *)malloc(length * sizeof(char));
        if(!set_cmd) {
            printf("注意：内存分配失败\r\n");
        }
        if (set_cmd == NULL) {
            printf("错误：内存分配失败\n");
            fclose(dbfile);
            return 1;
        }
        strncpy(set_cmd, set_start, length);
        set_cmd[length] = '\0'; // 添加字符串结束符

    } else {
        printf("错误：缺少SET指令\n");
        fclose(dbfile);
        return 1;
    }
    printf("SET %s\r\n" , set_cmd);

    /* 拆分每个SET */
    int set_cmd_count = StrChrShow(set_cmd, ',') + 1;   
    char *set_cmd_alone[set_cmd_count];
    set_cmd_alone[0] = strtok(set_cmd, ",");
    for(int i = 1; i < set_cmd_count; i++){
        set_cmd_alone[i] = strtok(NULL, ",");
        printf("strtok_SET %s\r\n" , set_cmd_alone[i]);
    }

    /* 拆分每个表达式的左右 */
    char *field_name[set_cmd_count];      // 字段名
    char *equal_value[set_cmd_count];     // 值
    for(int i = 0; i < set_cmd_count; i++){
        // 找到等号的位置
        char *equal = strchr(set_cmd_alone[i], '=');
        if (equal == NULL) {
            printf("错误：存在无意义update %s \n", set_cmd_alone[i]);
            continue;
        }

        // 提取字段名
        field_name[i] = (char *)malloc((equal - set_cmd_alone[i] + 1) * sizeof(char));
        if(!field_name[i]) {
            printf("注意：内存分配失败\r\n");
        }
        if (field_name[i] == NULL) {
            printf("错误：内存分配失败\n");
            fclose(dbfile);
            free(set_cmd);
            return 1;
        }
        strncpy(field_name[i], set_cmd_alone[i], equal - set_cmd_alone[i]);
        field_name[i][equal - set_cmd_alone[i]] = '\0'; // 添加字符串结束符

        printf("拆分每个表达式 %s\r\n" , field_name[i]);
        // 提取值
        equal += 1; // 跳过等号
        equal_value[i] = strdup(equal); // 复制等号后的内容
    }

    /* 提取表中的所有参数 */
    char *line_parameter[256]; 
    line_flag = 0;

    /* 读取表中数据内容 */
    while(fgets(line_str, sizeof(line_str), dbfile)) {
        char *line_temp = (char *)malloc(strlen(line_str) + 1);
        if(!line_temp) {
            printf("注意：内存分配失败\r\n");
        }
        if (line_temp == NULL) {
            printf("错误：内存分配失败\n");
            fclose(dbfile);
            free(set_cmd);
            for (int i = 0; i < set_cmd_count; i++) {
                free(field_name[i]);
                free(equal_value[i]);
            }
            return 1;
        }
        strcpy(line_temp, line_str);
        /* 去掉换行符 */
        line_str[strcspn(line_str, "\n")] = '\0';
        /* 读到END就结束了 */
        if(strcmp(line_str, "END") == 0) {
            // 保存当前位置
            fgetpos(dbfile, &pos_end);
            free(line_temp);
            break;
        }   

        char *token = strtok(line_str, " ");
        trim(token);

        /* 存储变量 */
        if(token != NULL) {
            line_parameter[line_flag] = strdup(token);
            if (strcmp(token, where_field) == 0) {
                where_field_index = line_flag;
            }
        }
        line_flag ++;
        free(line_temp);
    }

    /* 判断非法参数 */
    int field_point[set_cmd_count];
    memset(field_point, 0, sizeof(field_point));
    for(int x = 0; x < set_cmd_count; x++){
        int line_sign = 1;
        for(int i = 0; i < line_flag; i++){
            if(strstr(field_name[x], line_parameter[i]) != NULL){
                field_point[x] = i;
                line_sign = 0;
                break;
            }
        }
        if(line_sign){
            /* 存放循序倒置 */
            swap(&field_name[x], &equal_value[x]);
            line_sign = 1;
            for(int i = 0; i < line_flag; i++){
                if(strstr(field_name[x], line_parameter[i]) != NULL){
                    field_point[x] = i;
                    line_sign = 0;
                    break;
                }
            }
            if(line_sign){
                printf("错误:处理的参数表中不存在\n");
                /* 关闭文件 */
                fclose(dbfile);
                dbfile = NULL;

                // 释放动态分配的内存
                for (int i = 0; i < line_flag; i++) {
                    free(line_parameter[i]);
                }
                for (int i = 0; i < set_cmd_count; i++) {
                    free(field_name[i]);
                    free(equal_value[i]);
                }
                free(set_cmd);
                return 1;
            }
        }
    }   

    /* 读取表中数据内容,修改对应的元素之后再放进去 */
    fsetpos(dbfile, &pos_end);
    while (fgets(line_str, sizeof(line_str), dbfile) != NULL) {
        if (strcmp(line_str, "\n") == 0) {
            // 检测到空行，保存当前位置
            break;
        }

        int line_str_len = strlen(line_str);
        char temp[512];
        strcpy(temp, line_str);
        char *tokens[256];
        int token_count = 0;
        char *token = strtok(temp, "\\");
        while (token != NULL) {
            tokens[token_count++] = token;
            token = strtok(NULL, "\\");
        }

        // 检查是否满足WHERE条件
        if (where_clause && where_field_index != -1 && token_count > where_field_index) {
            trim(tokens[where_field_index]);
            if (strcmp(tokens[where_field_index], where_value) != 0) {
                continue;
            }
        }

        for(int i = 0; i < set_cmd_count; i++){
            if (field_point[i] < token_count) {
                strcpy(tokens[field_point[i]], equal_value[i]);
            }
        }

        char replacement[512] = "";
        for (int i = 0; i < token_count; i++) {
            if (i > 0) {
                strcat(replacement, "\\");
            }
            strcat(replacement, tokens[i]);
        }
        /* 去掉换行符 */
        replacement[strcspn(replacement, "\n")] = ' ';

        int replace_len = strlen(replacement);
        int final_len = line_str_len > replace_len ? line_str_len - replace_len : replace_len - line_str_len;
        for(int a = 0; a < final_len-1; a++)
            strcat(replacement, " ");
        // strcat(replacement, "\n");
        
        /* 将文件指针移回行首 */
        fseek(dbfile, -strlen(line_str), SEEK_CUR); 
        fprintf(dbfile, "%s\n", replacement);
        
    }

    /* 关闭文件 */
    fclose(dbfile);
    dbfile = NULL;

    // 释放动态分配的内存
    for (int i = 0; i < line_flag; i++) {
        free(line_parameter[i]);
    }
    for (int i = 0; i < set_cmd_count; i++) {
        free(field_name[i]);
        free(equal_value[i]);
    }
    free(set_cmd);

    return 0;
}
